@@ -185,6 +185,38 @@ G_GNUC_INTERNAL void _gio_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
+/* VOID:POINTER,INT,STRING (./gio-marshal.list:24) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__POINTER_INT_STRING (GClosure     *closure,
+                                                            GValue       *return_value,
+                                                            guint         n_param_values,
+                                                            const GValue *param_values,
+                                                            gpointer      invocation_hint,
+                                                            gpointer      marshal_data);
+
+/* BOOLEAN:OBJECT (./gio-marshal.list:25) */
+G_GNUC_INTERNAL void _gio_marshal_BOOLEAN__OBJECT (GClosure     *closure,
+                                                   GValue       *return_value,
+                                                   guint         n_param_values,
+                                                   const GValue *param_values,
+                                                   gpointer      invocation_hint,
+                                                   gpointer      marshal_data);
+
+/* INT:OBJECT (./gio-marshal.list:26) */
+G_GNUC_INTERNAL void _gio_marshal_INT__OBJECT (GClosure     *closure,
+                                               GValue       *return_value,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint,
+                                               gpointer      marshal_data);
+
+/* VOID:UINT64 (./gio-marshal.list:27) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__UINT64 (GClosure     *closure,
+                                                GValue       *return_value,
+                                                guint         n_param_values,
+                                                const GValue *param_values,
+                                                gpointer      invocation_hint,
+                                                gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* ___gio_marshal_MARSHAL_H__ */
