@@ -27,7 +27,6 @@
 #include "gloadableicon.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gloadableicon
@@ -48,55 +47,17 @@ static GInputStream *g_loadable_icon_real_load_finish (GLoadableIcon        *ico
 						       GAsyncResult         *res,
 						       char                **type,
 						       GError              **error);
-static void          g_loadable_icon_base_init        (gpointer              g_class);
-static void          g_loadable_icon_class_init       (gpointer              g_class,
-						       gpointer              class_data);
 
-GType
-g_loadable_icon_get_type (void)
-{
-  static GType loadable_icon_type = 0;
-
-  if (! loadable_icon_type)
-    {
-      static const GTypeInfo loadable_icon_info =
-	{
-        sizeof (GLoadableIconIface), /* class_size */
-	g_loadable_icon_base_init,   /* base_init */
-	NULL,		/* base_finalize */
-	g_loadable_icon_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	0,
-	0,              /* n_preallocs */
-	NULL
-      };
-
-      loadable_icon_type =
-	g_type_register_static (G_TYPE_INTERFACE, I_("GLoadableIcon"),
-				&loadable_icon_info, 0);
-
-      g_type_interface_add_prerequisite (loadable_icon_type, G_TYPE_ICON);
-    }
-
-  return loadable_icon_type;
-}
+typedef GLoadableIconIface GLoadableIconInterface;
+G_DEFINE_INTERFACE(GLoadableIcon, g_loadable_icon, G_TYPE_ICON)
 
 static void
-g_loadable_icon_class_init (gpointer g_class,
-			    gpointer class_data)
+g_loadable_icon_default_init (GLoadableIconIface *iface)
 {
-  GLoadableIconIface *iface = g_class;
-
   iface->load_async = g_loadable_icon_real_load_async;
   iface->load_finish = g_loadable_icon_real_load_finish;
 }
 
-static void
-g_loadable_icon_base_init (gpointer g_class)
-{
-}
-
 /**
  * g_loadable_icon_load:
  * @icon: a #GLoadableIcon.
@@ -109,7 +70,7 @@ g_loadable_icon_base_init (gpointer g_class)
  * Loads a loadable icon. For the asynchronous version of this function, 
  * see g_loadable_icon_load_async().
  * 
- * Returns: a #GInputStream to read the icon from.
+ * Returns: (transfer full): a #GInputStream to read the icon from.
  **/
 GInputStream *
 g_loadable_icon_load (GLoadableIcon  *icon,
@@ -165,7 +126,7 @@ g_loadable_icon_load_async (GLoadableIcon       *icon,
  * 
  * Finishes an asynchronous icon load started in g_loadable_icon_load_async().
  * 
- * Returns: a #GInputStream to read the icon from.
+ * Returns: (transfer full): a #GInputStream to read the icon from.
  **/
 GInputStream *
 g_loadable_icon_load_finish (GLoadableIcon  *icon,
@@ -277,6 +238,3 @@ g_loadable_icon_real_load_finish (GLoadableIcon        *icon,
 
   return g_object_ref (data->stream);
 }
-
-#define __G_LOADABLE_ICON_C__
-#include "gioaliasdef.c"
