@@ -2,7 +2,7 @@
 #include <unistd.h>
 #include <locale.h>
 #include <libintl.h>
-#include <gio.h>
+#include <gio/gio.h>
 #include <gstdio.h>
 
 typedef struct {
@@ -22,6 +22,7 @@ test_schema (gpointer data)
       gchar *path = g_build_filename (SRCDIR, "schema-tests", filename, NULL);
       gchar *argv[] = {
         "../glib-compile-schemas",
+        "--strict",
         "--dry-run",
         "--schema-file", path,
         (gchar *)test->opt,
@@ -101,8 +102,8 @@ static const SchemaTest tests[] = {
   { "extend-self",                  NULL, "*not yet existing*"                                  },
   { "extend-wrong-list-indirect",   NULL, "*'y' does not extend 'x'*"                           },
   { "extend-wrong-list",            NULL, "*'y' does not extend 'x'*"                           },
-  { "key-in-list-indirect",         NULL, "*can not add keys to a list*"                        },
-  { "key-in-list",                  NULL, "*can not add keys to a list*"                        },
+  { "key-in-list-indirect",         NULL, "*can not add keys to a 'list*"                        },
+  { "key-in-list",                  NULL, "*can not add keys to a 'list*"                        },
   { "list-of-missing",              NULL, "*is list of not yet existing schema*"                },
   { "extend-and-shadow",            NULL, "*shadows*use <override>*"                            },
   { "extend-and-shadow-indirect",   NULL, "*shadows*use <override>*"                            },
