@@ -14,6 +14,20 @@ static gboolean backend_set;
  * to be compiled and installed in the same directory.
  */
 
+static void
+check_and_free (GVariant    *value,
+                const gchar *expected)
+{
+  gchar *printed;
+
+  printed = g_variant_print (value, TRUE);
+  g_assert_cmpstr (printed, ==, expected);
+  g_free (printed);
+
+  g_variant_unref (value);
+}
+
+
 /* Just to get warmed up: Read and set a string, and
  * verify that can read the changed string back
  */
@@ -1560,7 +1574,7 @@ test_enums (void)
       if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
         g_settings_set_string (settings, "test", "qux");
       g_test_trap_assert_failed ();
-      g_test_trap_assert_stderr ("*g_settings_range_check*");
+      g_test_trap_assert_stderr ("*g_settings_key_info_range_check*");
 
       if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
         g_settings_get_flags (settings, "test");
@@ -1619,7 +1633,7 @@ test_flags (void)
         g_settings_set_strv (settings, "f-test",
                              (const gchar **) g_strsplit ("rock", ",", 0));
       g_test_trap_assert_failed ();
-      g_test_trap_assert_stderr ("*g_settings_range_check*");
+      g_test_trap_assert_stderr ("*g_settings_key_info_range_check*");
 
       if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
         g_settings_get_enum (settings, "f-test");
@@ -1678,12 +1692,12 @@ test_range (void)
       if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
         g_settings_set_int (settings, "val", 45);
       g_test_trap_assert_failed ();
-      g_test_trap_assert_stderr ("*g_settings_range_check*");
+      g_test_trap_assert_stderr ("*g_settings_key_info_range_check*");
 
       if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
         g_settings_set_int (settings, "val", 1);
       g_test_trap_assert_failed ();
-      g_test_trap_assert_stderr ("*g_settings_range_check*");
+      g_test_trap_assert_stderr ("*g_settings_key_info_range_check*");
     }
 
   g_assert_cmpint (g_settings_get_int (settings, "val"), ==, 33);
@@ -1767,12 +1781,17 @@ static void
 test_list_schemas (void)
 {
   const gchar * const *schemas;
+  const gchar * const *relocs;
 
+  relocs = g_settings_list_relocatable_schemas ();
   schemas = g_settings_list_schemas ();
 
+  g_assert (strv_set_equal ((gchar **)relocs,
+                            "org.gtk.test.no-path",
+                            NULL));
+
   g_assert (strv_set_equal ((gchar **)schemas,
                             "org.gtk.test",
-                            "org.gtk.test.no-path",
                             "org.gtk.test.basic-types",
                             "org.gtk.test.complex-types",
                             "org.gtk.test.localized",
@@ -1838,6 +1857,34 @@ test_get_mapped (void)
   g_object_unref (settings);
 }
 
+static void
+test_get_range (void)
+{
+  GSettings *settings;
+  GVariant *range;
+
+  settings = g_settings_new ("org.gtk.test.range");
+  range = g_settings_get_range (settings, "val");
+  check_and_free (range, "('range', <(2, 44)>)");
+  g_object_unref (settings);
+
+  settings = g_settings_new ("org.gtk.test.enums");
+  range = g_settings_get_range (settings, "test");
+  check_and_free (range, "('enum', <['foo', 'bar', 'baz', 'quux']>)");
+  g_object_unref (settings);
+
+  settings = g_settings_new ("org.gtk.test.enums");
+  range = g_settings_get_range (settings, "f-test");
+  check_and_free (range, "('flags', "
+                  "<['mourning', 'laughing', 'talking', 'walking']>)");
+  g_object_unref (settings);
+
+  settings = g_settings_new ("org.gtk.test");
+  range = g_settings_get_range (settings, "greeting");
+  check_and_free (range, "('type', <@as []>)");
+  g_object_unref (settings);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1921,6 +1968,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/gsettings/list-items", test_list_items);
   g_test_add_func ("/gsettings/list-schemas", test_list_schemas);
   g_test_add_func ("/gsettings/mapped", test_get_mapped);
+  g_test_add_func ("/gsettings/get-range", test_get_range);
 
   result = g_test_run ();
 
