@@ -69,7 +69,7 @@
 #include "gstrfuncs.h"
 #include "gtestutils.h"
 #include "gthread.h"
-#include "gtimezoneprivate.h"
+#include "gtimezone.h"
 
 #include "glibintl.h"
 
@@ -481,7 +481,7 @@ g_date_time_from_instant (GTimeZone *tz,
   GDateTime *datetime;
   gint64 offset;
 
-  if (instant < 0 || instant > 1000000000000000000)
+  if (instant < 0 || instant > G_GINT64_CONSTANT (1000000000000000000))
     return NULL;
 
   datetime = g_date_time_alloc (tz);
@@ -1584,7 +1584,7 @@ g_date_time_get_day_of_month (GDateTime *datetime)
 /* Week of year / day of week getters {{{1 */
 /**
  * g_date_time_get_week_numbering_year:
- * @date: a #GDateTime
+ * @datetime: a #GDateTime
  *
  * Returns the ISO 8601 week-numbering year in which the week containing
  * @datetime falls.
