@@ -20,6 +20,10 @@
  *   Boston, MA 02111-1307, USA.
  */
 
+#if defined(G_DISABLE_SINGLE_INCLUDES) && !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
+#error "Only <glib.h> can be included directly."
+#endif
+
 #ifndef __G_KEY_FILE_H__
 #define __G_KEY_FILE_H__
 
@@ -132,6 +136,22 @@ void      g_key_file_set_integer            (GKeyFile             *key_file,
 					     const gchar          *group_name,
 					     const gchar          *key,
 					     gint                  value);
+gint64    g_key_file_get_int64              (GKeyFile             *key_file,
+					     const gchar          *group_name,
+					     const gchar          *key,
+					     GError              **error);
+void      g_key_file_set_int64              (GKeyFile             *key_file,
+					     const gchar          *group_name,
+					     const gchar          *key,
+					     gint64                value);
+guint64   g_key_file_get_uint64             (GKeyFile             *key_file,
+					     const gchar          *group_name,
+					     const gchar          *key,
+					     GError              **error);
+void      g_key_file_set_uint64             (GKeyFile             *key_file,
+					     const gchar          *group_name,
+					     const gchar          *key,
+					     guint64               value);
 gdouble   g_key_file_get_double             (GKeyFile             *key_file,
                                              const gchar          *group_name,
                                              const gchar          *key,
@@ -192,7 +212,7 @@ void      g_key_file_set_integer_list       (GKeyFile             *key_file,
 					     const gchar          *key,
 					     gint                  list[],
 					     gsize                 length);
-void      g_key_file_set_comment            (GKeyFile             *key_file,
+gboolean  g_key_file_set_comment            (GKeyFile             *key_file,
                                              const gchar          *group_name,
                                              const gchar          *key,
                                              const gchar          *comment,
@@ -202,15 +222,15 @@ gchar    *g_key_file_get_comment            (GKeyFile             *key_file,
                                              const gchar          *key,
                                              GError              **error) G_GNUC_MALLOC;
 
-void      g_key_file_remove_comment         (GKeyFile             *key_file,
+gboolean  g_key_file_remove_comment         (GKeyFile             *key_file,
                                              const gchar          *group_name,
                                              const gchar          *key,
 					     GError              **error);
-void      g_key_file_remove_key             (GKeyFile             *key_file,
+gboolean  g_key_file_remove_key             (GKeyFile             *key_file,
 					     const gchar          *group_name,
 					     const gchar          *key,
 					     GError              **error);
-void      g_key_file_remove_group           (GKeyFile             *key_file,
+gboolean  g_key_file_remove_group           (GKeyFile             *key_file,
 					     const gchar          *group_name,
 					     GError              **error);
 
