@@ -363,8 +363,12 @@ void     g_source_add_poll         (GSource        *source,
 void     g_source_remove_poll      (GSource        *source,
                                     GPollFD        *fd);
 
+#ifndef G_DISABLE_DEPRECATED
 void     g_source_get_current_time (GSource        *source,
                                     GTimeVal       *timeval);
+#endif
+void     g_source_get_time         (GSource        *source,
+                                    GTimeSpec      *timespec);
 
  /* void g_source_connect_closure (GSource        *source,
                                   GClosure       *closure);
@@ -380,6 +384,7 @@ GSource *g_timeout_source_new_seconds (guint interval);
 /* Miscellaneous functions
  */
 void g_get_current_time                 (GTimeVal       *result);
+void g_get_monotonic_time               (GTimeSpec      *result);
 
 /* ============== Compat main loop stuff ================== */
 
